


/*
Combat - A tank simulation for X written in C++

Copyright (C) 1997,1998 Robert Muth <muth@cs.arizona.edu>

This program is free software; you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation; version 2 (June 1991)

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, 
write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


//============================================================
// CONSOLE.CC
// ROBERT MUTH
// 08.03.96
//============================================================

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/Xos.h>

#include <string.h>
#include <stream.h>

#include "console.hh"

#include "combat.icon.h"

//============================================================
// CONSTANTS 
//============================================================

#define FONTNAME  "-bitstream-charter-bold-r-normal-*-40-*-*-*-p-*-*-*"
#define MY_EVENT_MASK \
(KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask|ExposureMask)

//============================================================
// HELPER FUNCTIONS
//============================================================

int GetColorByName(Display *d, int s, char *c)
{
    XColor          color;
    Colormap        cmap;
    
    cmap = DefaultColormap(d, s);
    
    if( !XParseColor (d, cmap, c, &color)) 
    {
        cerr << "cannot find color: " << c << endl;
    }
    
    if( !XAllocColor(d, cmap, &color)) 
    {
        cerr << "cannot allocate color: " << c << endl;
    }

    return color.pixel;
}

//============================================================
// DESTRUCTOR
//============================================================

Console::~Console()
{
    for(int i=0; i<nColors; i++)
        XFreeGC(display, gc[i]);

    XFreeFont(display, font);
    XDestroyWindow(display, window);
    XDestroyWindow(display, bitmap);    
}

//============================================================
// CONSTRUCTOR
//============================================================

Console::Console(char *displaystring, int w, int h)
{
    XGCValues       values;
    XWMHints        wmhints;
    XSizeHints      sh;

    display = XOpenDisplay(displaystring);

    if( display == 0 )
    {
        cerr << "Cannot open display: " << displaystring << endl;
        exit(-1);
    }
    
    screen = DefaultScreen(display);
    depth = DefaultDepth(display, screen);
    
    window = XCreateSimpleWindow( display, 
                                  DefaultRootWindow(display), 
                                  0, 0, w, h, 
                                  0,
                                  WhitePixel( display, screen),
                                  BlackPixel( display, screen));
        
    if( window == 0 )
    {
        cerr << "Cannot open window: " <<  displaystring << endl;
        exit(-1);
    }

    wmhints.initial_state = NormalState;
    wmhints.input = True;
    wmhints.icon_pixmap = XCreateBitmapFromData( display, window, combat_bits,
                                                 combat_width, combat_height);

    wmhints.flags = StateHint | InputHint | IconPixmapHint;
    
    sh.flags = PSize | PMinSize | PMaxSize;
    sh.width = sh.min_width = sh.max_width = w;
    sh.height = sh.min_height = sh.max_height = h;
    
    XSetWMProperties( display, window, 
		      NULL, NULL, NULL, 0, &sh, &wmhints, NULL);
    
    XSelectInput(display, window, MY_EVENT_MASK);
    
    bitmap = XCreatePixmap( display, window, w, h, depth);
    
    if( bitmap == 0 )
    {
        cerr << "Cannot open bitmap: " <<  displaystring << endl;
        exit(-1);
    }
    
    font = XLoadQueryFont(display, FONTNAME);

    if( font == 0 )
    {
        cerr << "Cannot Load Font: " <<  FONTNAME << endl;
        exit(-1);
    }
    
    values.background = BlackPixel( display, screen );
    values.graphics_exposures = False;
    values.font = font->fid;
    
    values.foreground = WhitePixel( display, screen );
    gc[Default] = XCreateGC( display, bitmap, GCFont | GCForeground | GCBackground |
                           GCGraphicsExposures, &values);
    
    values.foreground = GetColorByName(display,screen,"black"); 
    gc[Black] = XCreateGC( display, bitmap, GCFont | GCForeground | GCBackground |
                         GCGraphicsExposures, &values);
        
    values.foreground = GetColorByName(display,screen,"white"); 
    gc[White] = XCreateGC (display, bitmap, GCFont | GCForeground | GCBackground |
                         GCGraphicsExposures, &values);

    values.foreground = GetColorByName(display,screen,"blue"); 
    gc[Blue] = XCreateGC (display, bitmap, GCFont | GCForeground | GCBackground |
                        GCGraphicsExposures, &values);

    values.foreground = GetColorByName(display,screen,"red"); 
    gc[Red] = XCreateGC (display, bitmap, GCFont | GCForeground | GCBackground |
                       GCGraphicsExposures, &values);

    values.foreground = GetColorByName(display,screen,"green"); 
    gc[Green] = XCreateGC (display, bitmap, GCFont | GCForeground | GCBackground |
                         GCGraphicsExposures, &values);

    XMapWindow(display,window);
}

//============================================================
// DRAWING PRIMITIVES
//============================================================

void Console::DrawText(char *text, int length, double x, double y, Color color)
{
    XDrawString(display, bitmap, gc[color], int(x), int(y), text, length);
}

void Console::DrawLine(double x1, double y1, double x2, double y2, Color color)
{
    XDrawLine( display, bitmap, gc[color], int(x1), int(y1), int(x2), int(y2));
}

void Console::DrawRectangle(double x, double y, double w, double h, Color color)
{
    XFillRectangle( display, bitmap, gc[color], int(x), int(y), int(w), int(h));
}

void Console::DrawFilledCircle(double x, double y, double r, Color color)
{
    XFillArc( display, bitmap, gc[color], int(x - r) , int(y - r), 
              int(r + r + 1), int(r + r + 1), 0, 360*64);
}

void Console::DrawCircle(double x, double y, double r, Color color)
{
    XDrawArc( display, bitmap, gc[color], int(x - r + 1) , int(y - r + 1), 
              int(r + r - 1), int(r + r - 1), 0, 360*64);
}

void Console::UpdateArea(double x, double y, double w, double h)
{
    XCopyArea( display, bitmap, window, gc[Default], 
               int(x), int(y), int(w), int(h), int(x), int(y));
}

//============================================================
// SET WINDOW TITLE
//============================================================

void Console::SetTitle(char *text)
{
    XStoreName (display, window, text);
}

//============================================================
// READ EVENT
//============================================================

Action Console::GetEvent()
{
    XEvent          ev;
    
    if( !XPending(display) )
        return NoAction;

    XNextEvent(display,&ev);
    
    switch (ev.type) 
    {
    case Expose:
        //        cerr << "Expose" << endl;
        XCopyArea( display, bitmap, window, gc[Default], 
                       ev.xexpose.x,ev.xexpose.y,ev.xexpose.width,ev.xexpose.height,
                       ev.xexpose.x,ev.xexpose.y);
        if (ev.xexpose.count == 0)
            XFlush(display);
        break;
            
    case KeyPress:
        //        cerr << "Key Release" << endl;
        switch( XLookupKeysym(&ev.xkey, 0) & 0x7f )
        {
        case 'q':
            return Quit;
        case 'x':
            return Place;
        case ' ':
            return Fire;
        case 'z':
            return HideOn;
        case 'j':
        case '4':
            return LeftOn;
        case '8':
        case 'i':
            return ForwardOn;
        case 'l':
        case '6':
            return RightOn;
        case 'm':
        case '2':
            return BackwardOn;
        default:
            break;
        }
        break;
        
        
    case KeyRelease:		
        //        cerr << "Key Release" << endl;
        switch( XLookupKeysym(&ev.xkey, 0) & 0x7f )
        {
        case 'z':
            return HideOff;
        case 'j':
        case '4':
            return LeftOff;
        case '8':
        case 'i':
            return ForwardOff;
        case 'l':
        case '6':
            return RightOff;
        case '2':
        case 'm':
            return BackwardOff;
        default:
            break;
        }
        break;
        
    case DestroyNotify:
        //        cerr << "Destroy Notification" << endl;
        return Quit;
	break;
        
    default:
        break;
    }

    return GetEvent();
}

void Console::Sync()
{
    XSync(display,False);
}


