


/*
Combat - A tank simulation for X written in C++

Copyright (C) 1997,1998 Robert Muth <muth@cs.arizona.edu>

This program is free software; you can redistribute it and/or modify it under the terms of 
the GNU General Public License as published by the Free Software Foundation; version 2 (June 1991)

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, 
write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


//============================================================
// MAIN.CC
//============================================================


#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>

#include <stream.h>

#include "combat.hh"
#include "player.hh"
#include "object.hh"


//============================================================
// GLOBALs
//============================================================

struct
{
    long RandomSeed;
    bool DebugFlag;
    int  Width;
    int  Height;
    int  Delay;
    int  nRocks;
    int  nPlayers;
    Player *PlayerList[256];
}
Global =
{ 
    0, false, 800, 600, DELAY, NROCKS, 0
};

ObjectStack RockStack;
ObjectStack TankStack;
ObjectStack ShotStack;

//============================================================
// AUXILIARIES
//============================================================

#define tab "\t"

int usage()
{
    cerr << "combat [-r nrocks] [-d] [-s seed] [-w delay] display1 display2 ..." << endl
         << tab << "Version: 1.00" << endl
         << tab << "-d        debugging modes" << endl
         << tab << "-r rocks  number of rocks" << endl
         << tab << "-s seed   initial seed for random number generator" << endl
         << tab << "-w delay  delay in usecs (default 50000)" << endl;
    
    return -1;
}


void mysleep(int sec, int usec)
{
    //    struct timespec x = { secs, nanosecs };
    //
    //    while( 0 != nanosleep(&x, &x) )
    //        ;

    fd_set readset;
    struct timeval tv;

    FD_ZERO (&readset);
    tv.tv_sec = sec;
    tv.tv_usec = usec;
    select (0, &readset, NULL, NULL, &tv);
}

//============================================================
// PLACE TANK
//============================================================

Tank *PlaceTank(Player *player)
{
    Tank *tank;

    Rectangle Playfield(0,0,Global.Width,Global.Height);
    
    do
    {
        tank = new Tank( Global.Width * drand48(), Global.Height * drand48(),
                            2.0 * M_PI * drand48(), player);
    }    
    while( !tank->Contained(0,0,Global.Width,Global.Height) ||
           -1 != TankStack.FindCollisions(tank) ||
           -1 != RockStack.FindCollisions(tank) ||
           -1 != ShotStack.FindCollisions(tank) );
    
    TankStack.Push(tank);
    Playfield.Draw(player);
    RockStack.Draw(player);
    return tank;
}


//============================================================
// FIRE SHOT
//============================================================

Shot *FireShot(Tank *tank)
{
    Shot *shot = new Shot(*tank);
    ShotStack.Push( shot );
    return shot;
}

              
//============================================================
// LOOP
//============================================================

void Loop()
{
    int i;

    Rectangle Playfield(0,0,Global.Width,Global.Height);

    Text GameOver("GAME OVER",9,300,200);
    Text JoinTheGame("JOIN THE GAME",13,250,200);
    Text YouWin("YOU WIN",7,300,200);

   
    for( i=0; i<Global.nPlayers; i++ )
    {
        Playfield.Draw(Global.PlayerList[i]);
        RockStack.Draw(Global.PlayerList[i]);
    }            

    while( true )
    {
        //============================================================        
        // DRAW ALL REALLY MOVING OBJECTS
        //============================================================

        if( Global.DebugFlag ) cerr << "Draw Objects" << endl;
        
        for( i=0; i<Global.nPlayers; i++ )
        {
            TankStack.Draw(Global.PlayerList[i]);
            ShotStack.Draw(Global.PlayerList[i]);
            Global.PlayerList[i]->UpdateArea(0,0,Global.Width,Global.Height);
        }

        //============================================================        
        // UNDRAW ALL REALLY MOVING OBJECTS
        //============================================================

        if( Global.DebugFlag ) cerr << "UnDraw Objects" << endl;
                
        for( i=0; i<Global.nPlayers; i++ )
        {
            ShotStack.Undraw(Global.PlayerList[i]);
            TankStack.Undraw(Global.PlayerList[i]);
        }

        //============================================================        
        // HANDLE EVENTS
        //============================================================

        if( Global.DebugFlag ) cerr << "Handle Events" << endl;
        
        for( i=0; i<Global.nPlayers; i++ )
        {
            Global.PlayerList[i]->EventHandler();
        }

        //============================================================        
        // DRAW TEXTS
        //============================================================

        if( Global.DebugFlag ) cerr << "Draw Texts" << endl;
        
        int alivePlayers = 0;
        
        for( i=0; i<Global.nPlayers; i++ )
        {
            if( Global.PlayerList[i]->isAlive() )
            {
                alivePlayers++;
                if( !Global.PlayerList[i]->isActive() )
                    JoinTheGame.Draw(Global.PlayerList[i]);
            }
            else
            {
                GameOver.Draw(Global.PlayerList[i]);                
            }
        }

        if( alivePlayers <= 1 )
        {
            for( i=0; i<Global.nPlayers; i++ )
            {
                if( Global.PlayerList[i]->isAlive() )
                    YouWin.Draw(Global.PlayerList[i]);
                Global.PlayerList[i]->UpdateArea(0,0,Global.Width,Global.Height);
                Global.PlayerList[i]->Sync();
            }
            
            mysleep(5,0);
            return; // <<<<<<<<<<<<<<<<<
        }

        //============================================================        
        // MOVE SHOTS AND TANKS
        //============================================================

        if( Global.DebugFlag ) cerr << "Move Objects" << endl;
            
        for(i=0; i<ShotStack.Size(); i++)
        {
            Object *shot = ShotStack.Peek(i);

            shot->Move();
        }

        for(i=0; i<TankStack.Size(); i++)
        {
            Object *tank = TankStack.Peek(i);

            tank->Move();

            if( !tank->Contained(0,0,Global.Width,Global.Height) ||
                -1 != TankStack.FindCollisions(tank) ||
                -1 != RockStack.FindCollisions(tank) )
                tank->Unmove();
        }

        //============================================================        
        // COLLISION DETECTION
        //============================================================

        for(i=0; i<ShotStack.Size(); i++)
        {
            Object *shot = ShotStack.Peek(i);
            
            if( !shot->Contained(0,0,Global.Width,Global.Height) )
            {
                shot->Creator()->ShotKilled();
                ShotStack.Kill(i);
                i--;
                continue;
            }

            
            if( -1 != RockStack.FindCollisions( shot ) )
            {
                shot->Creator()->ShotKilled();
                ShotStack.Kill(i);
                i--;
                continue;
            }

            int t;
            
            if( -1 !=  (t=TankStack.FindCollisions( shot )) )
            {
                TankStack.Peek(t)->Creator()->TankKilled();
                TankStack.Kill(t);
                shot->Creator()->ShotKilled();                
                ShotStack.Kill(i);

                i--;
                continue;
            }
            
        }

        //============================================================        
        // SYNC
        //============================================================

        mysleep(0,Global.Delay);
    }
    
}

//============================================================
// MAIN
//============================================================

int main(int argc, char *argv[] )
{
    
    int i;

    while( EOF != (i = getopt(argc,argv,"r:s:dw:")) )
    {
        switch(i)
        {
        case 'w':
	    Global.Delay = atoi(optarg);
            break;
        case 'r':
	    Global.nRocks = atoi(optarg);
            break;
	case 'd':
	    Global.DebugFlag = true;
	    break;
        case 's':
	    Global.RandomSeed = atoi(optarg);
	    break;
        case '?':
        case 'h':
	default:
            return usage();
            break;
	}
    }

    argc -= optind;
    argv += optind;
    
    srand48(Global.RandomSeed);    


    //============================================================
    // CREATE ROCKS
    //============================================================
    
    if( Global.DebugFlag ) cerr << "Creating Rocks" << endl;
        
    if( Global.RandomSeed == 0 && Global.nRocks == 9 )
    {
        RockStack.Push( new Rock(150,150,70) );
        RockStack.Push( new Rock(150,450,70) );
        RockStack.Push( new Rock(650,150,70) );
        RockStack.Push( new Rock(650,450,70) );

        RockStack.Push( new Rock(400,300,50) );
        RockStack.Push( new Rock(400,380,50) );
        RockStack.Push( new Rock(400,220,50) );
        RockStack.Push( new Rock(320,300,50) );

        RockStack.Push( new Rock(480,300,50) );
    }
    else for( i=0; i<Global.nRocks; i++ )
    {
        RockStack.Push( new Rock( Global.Width * drand48(),
                                Global.Height * drand48(),
                                Global.Height / 6 * drand48() ));
    }
    
    //============================================================
    // CREATE PLAYERS
    //============================================================

    if( argc == 0)
        return  usage();

    Global.nPlayers = argc;
    
    if( Global.DebugFlag ) cerr << "Creating Players" << endl;
    
    for( i=0; i<Global.nPlayers; i++ )
    {
        Global.PlayerList[i] = new Player(argv[i],Global.Width,Global.Height);
    }


    //============================================================
    // LOOP
    //============================================================

    if( Global.DebugFlag ) cerr << "Entering Main Loop" << endl;
    
    Loop();
    
    return 0;
}

//============================================================
// EOF
//============================================================







