//============================================================
// OBJECT.HH
// ROBERT MUTH
// 08.03.96
//============================================================


#ifndef OBJECT_H
#define OBJECT_H

#include "player.hh"

//============================================================
// PLAYFIELD
//============================================================
       
class Rectangle
{
public:    
	double x,y,w,h;

	Rectangle(double a, double b, double c, double d) : 
            x(a), y(b), w(c), h(d) {}
        
	void Draw(Player *p)
            {
                p->DrawRectangle(x, y, w, h, Green);
            }
};

//============================================================
// TEXT
//============================================================

class Text 
{
protected:
    char *text;
    int length;
    double x,y;
public:
    Text(char *t, int l, double a, double b) :
        text(t), length(l), x(a), y(b) {}
         
    void Draw(Player *p)
    {
        p->DrawText(text, length, x, y, Red);
    }
};

//============================================================
//  OBJECT
//============================================================

class Object 
{
protected:
    double xpos,ypos,radius,angle;
    Player *creator;
public:
    Object(double x, double y, double r, double a, Player *p) :
        xpos(x), ypos(y), radius(r), angle(a), creator(p) {}

    Player *Creator()
        {
            return creator;
        }

    virtual void Move() = 0;
    virtual void Unmove() = 0;

    virtual void Draw(Player *p) = 0;
    
    void Undraw(Player *p)
    {
        p->DrawFilledCircle(xpos,ypos,radius,Green);
    }
    
    bool Contained( double x, double y, double w, double h)
        {
            if( xpos - radius < x ) return false;
            if( ypos - radius < y ) return false;
            if( xpos + radius > x + w ) return false;
            if( ypos + radius > y + h ) return false;
            return true;
        }

    friend bool Intersect(Object *obj1, Object *obj2)
	{
	    double x = obj1->xpos - obj2->xpos;
	    double y = obj1->ypos - obj2->ypos;
	    double r = obj1->radius + obj2->radius;
            
	    x *= x, y *= y, r *= r;
            
            if( r > x + y )
                 return true;
            else
                 return false;
	}
};

//============================================================
// OBJECT STACK 
//============================================================

class ObjectStack
{
protected:
    Object *ObjectList[256];
    int     nObjects;
public:

    ObjectStack() : nObjects(0) {}

    Object *Peek(int pos)
        {
            return ObjectList[pos];
        }
    
    int Size()
        {
            return nObjects;
        }
    
    
    int Push(Object *obj)
        {
            ObjectList[nObjects++] = obj;
            return nObjects-1;
        }
    
    void Kill(int pos)
        {
            delete ObjectList[pos];
            ObjectList[pos] = ObjectList[--nObjects];
        }

    void Draw(Player *p)
        {
            for(int i=0; i<nObjects; i++)
                ObjectList[i]->Draw(p);
        }

    void Undraw(Player *p)
        {
            for(int i=0; i<nObjects; i++)
//            for(int i=nObjects-1; i>=0; i--)
                ObjectList[i]->Undraw(p);
        }

    int FindCollisions(Object *obj)
        {
            for( int i=0; i<nObjects; i++ )
                if( ObjectList[i]->Creator() != obj->Creator() &&
                    Intersect(ObjectList[i],obj) )
                return i;
            return -1;
        }
    
};

//============================================================
// ROCK 
//============================================================

class Rock : public Object
{
public:

    Rock(double x, double y, double r) : Object(x,y,r,0,0) {}

    void Draw(Player *p)
    {
        p->DrawFilledCircle(xpos, ypos, radius, Black);
    }
    
    void Move() {}
    void Unmove() {}    
};

//============================================================
// TANK
//============================================================

class Tank : public Object
{
public:
    Tank(double x, double y, double a, Player *p) :
        Object(x,y,TANK_RADIUS,a,p) {}

    void Draw(Player *p)
    {
        if( p == Creator() )
        {
            if( Creator()->isHidden() )
            {
                p->DrawCircle(xpos, ypos, radius, Blue);
                p->DrawLine(xpos, ypos,
                        xpos + cos(angle) * (radius-1),
                        ypos + sin(angle) * (radius-1), Blue);
                p->DrawFilledCircle(xpos, ypos, radius/3, Blue);
            }
            else
            {
                p->DrawFilledCircle(xpos, ypos, radius, Blue);
                p->DrawLine(xpos, ypos,
                        xpos + cos(angle) * (radius-1),
                        ypos + sin(angle) * (radius-1), White);
                p->DrawFilledCircle(xpos, ypos, radius/3, White);
            }
        }
        else
        {
            if( Creator()->isHidden() )
            {
                // no display of hidden players
            }
            else
            {
                p->DrawFilledCircle(xpos, ypos, radius, Red);
                p->DrawLine(xpos, ypos,
                        xpos + cos(angle) * (radius-1),
                        ypos + sin(angle) * (radius-1), White);
                p->DrawFilledCircle(xpos, ypos, radius/3, White);
            }
        }
    }

    void Move()
    {
        if( Creator()->turnLeft() )
            angle -= 2.0 * M_PI / 64.0;
        
        if( Creator()->turnRight() )
            angle += 2.0 * M_PI / 64.0;
           
        if( Creator()->moveForward() )
        {
            xpos += cos(angle) * TANK_SPEED ;
            ypos += sin(angle) * TANK_SPEED ;
        }

        if(  Creator()->moveBackward() )
        {
            xpos -= cos(angle) * TANK_SPEED ;
            ypos -= sin(angle) * TANK_SPEED ;
        }
    }

    void Unmove()
    {
        if( Creator()->moveForward() )
        {
            xpos -= cos(angle) * TANK_SPEED ;
            ypos -= sin(angle) * TANK_SPEED ;
        }

        if(  Creator()->moveBackward() )
        {
            xpos += cos(angle) * TANK_SPEED ;
            ypos += sin(angle) * TANK_SPEED ;
        }
    }    
};

//============================================================
// SHOT
//============================================================

class Shot : public Object
{
public:

    Shot(double x, double y, double r, double a, Player *p) :
        Object(x,y,r,a,p) {}

    Shot( Object& obj ) : Object( obj ) 
        {
            radius = SHOT_RADIUS;
        }

    void Draw(Player *p )
        {
            p->DrawFilledCircle(xpos, ypos, radius, p == Creator() ? Blue : Red);
        }

    void Move()
    {
        xpos += cos(angle) * SHOT_SPEED ;
        ypos += sin(angle) * SHOT_SPEED ;
    }

    void Unmove()
    {
        xpos -= cos(angle) * SHOT_SPEED ;
        ypos -= sin(angle) * SHOT_SPEED ;
    }    
};


#endif // OBJECT_H





